from qgis.PyQt.QtCore import (QT_TRANSLATE_NOOP)
from qgis.core import (
  QgsProcessingParameterNumber,
  QgsProcessingParameterFeatureSource,
  QgsProcessing,
  QgsProcessingParameterField,
  QgsProcessingParameterFolderDestination,
  QgsFeatureRequest,
  QgsProcessingParameterBoolean,
  QgsCoordinateReferenceSystem,
  QgsProject
  )

from .fetchabstract import fetchabstract

import os
import glob

class fetchtengunlist(fetchabstract):
  
  # UIs
  PARAMETERS = {  
    "DATA_LIST_LAYER": {
      "ui_func": QgsProcessingParameterFeatureSource,
      "ui_args":{
        "optional": True,
        "description": QT_TRANSLATE_NOOP("fetchtengunlist","List of data to be fetched"),
        "types": [QgsProcessing.TypeVectorPolygon, QgsProcessing.TypeVectorLine, QgsProcessing.TypeVectorPoint]
      }
    },
    "URL_FIELD": {
      "ui_func": QgsProcessingParameterField,
      "ui_args":{
        "description": QT_TRANSLATE_NOOP("fetchtengunlist","URL Field of the data list layer"),
        "parentLayerParameterName": "DATA_LIST_LAYER",
        "defaultValue": "url"
      }
    },
    "CRS_FIELD": {
      "ui_func": QgsProcessingParameterField,
      "ui_args":{
        "optional": True,
        "description": QT_TRANSLATE_NOOP("fetchtengunlist","CRS Field of the data list layer"),
        "parentLayerParameterName": "DATA_LIST_LAYER",
        "defaultValue": "crs"
      }
    },
    "MAX_NUM_FILES": {
      "ui_func": QgsProcessingParameterNumber,
      "ui_args": {
        "type": QgsProcessingParameterNumber.Integer,
        "description": QT_TRANSLATE_NOOP("fetchtengunlist","Maximum number of files to be downloaded"),
        "defaultValue": 10
      }
    },       
    "ADD_PC_TO_CURRENT_PROJ": {
      "ui_func": QgsProcessingParameterBoolean,
      "ui_args": {
        "description": QT_TRANSLATE_NOOP("fetchtengunextent","Add point cloud(s) to the current project?"),
        "defaultValue": True
      }
    },       
    "OUTPUT_FOLDER": {
      "ui_func": QgsProcessingParameterFolderDestination,
      "ui_args": {
        "description": QT_TRANSLATE_NOOP("fetchtengunlist","Folder to output")
      }
    }
  }
    
  OUTPUT_FOLDER = ""
    
  # initialization of the algorithm
  def initAlgorithm(self, config):    
    self.initParameters()
  
  # set the arguments
  def setWebFetchArgs(self, parameters, context, feedback):
    
    self.WEBFETCH_ARGS["REQUEST_ARGS"] = []
    data_list = self.parameterAsSource(parameters, "DATA_LIST_LAYER", context).materialize(QgsFeatureRequest(), feedback)
    url_field = self.parameterAsString(parameters, "URL_FIELD", context)
    crs_field = self.parameterAsString(parameters, "CRS_FIELD", context)
    
    for ft in data_list.getFeatures():
      self.WEBFETCH_ARGS["REQUEST_ARGS"].append(
        {
          "method": "GET",
          "url": ft[url_field],
          "crs": QgsCoordinateReferenceSystem(ft[crs_field]) if crs_field is not None else None
        }
      )
    
    
    if len(self.WEBFETCH_ARGS["REQUEST_ARGS"]) > self.parameterAsInt(parameters, "MAX_NUM_FILES", context):
      feedback.reportError(self.tr("Number of files exceeds: ") + str(self.parameterAsInt(parameters, 'MAX_NUM_FILES', context)))
      raise Exception(self.tr("Number of files exceeds: ") + str(self.parameterAsInt(parameters, 'MAX_NUM_FILES', context)))
    
    if self.WEBFETCH_ARGS["REQUEST_ARGS"] is not None:
      self.WEBFETCH_ARGS["SET"] = True
    
  
  # execution of the algorithm
  def processAlgorithm(self, parameters, context, feedback):        
    # import ptvsd
    # ptvsd.debug_this_thread()
    # set output folder
    self.OUTPUT_FOLDER = os.path.normpath(self.parameterAsString(parameters, "OUTPUT_FOLDER", context))
    if len(self.OUTPUT_FOLDER) != len(self.OUTPUT_FOLDER.encode('utf-8')):
      feedback.reportError(self.tr("The output folder path must not contain multi-byte characters."))
      raise Exception(self.tr("The output folder path must not contain multi-byte characters."))
    if not os.path.exists(self.OUTPUT_FOLDER):
      os.makedirs(self.OUTPUT_FOLDER)
    
    # set the meta information for obtaining tengun data
    self.setWebFetchArgs(parameters, context, feedback)
        
    # download tengun files, in self.TENGUN_FILES
    self.fetchMultipleTenguns(parameters, context, feedback, parallel=False, output_dir=self.OUTPUT_FOLDER)
          
    if self.parameterAsBoolean(parameters, "ADD_PC_TO_CURRENT_PROJ", context):
      
      for tng in glob.glob(os.path.join(self.OUTPUT_FOLDER, "*")):
        context.addLayerToLoadOnCompletion(
          tng, 
          context.LayerDetails(os.path.splitext(os.path.basename(tng))[0], QgsProject.instance(), "")
        )
    
    return {"OUTPUT": self.OUTPUT_FOLDER}
  
  def displayName(self):
    return self.tr("Fetch Point Cloud using list")

  def createInstance(self):
    return fetchtengunlist()
